//
//  AdvancedDialog.cpp : Implementation file for class AdvancedDialog
//
//  CubeShow Version 1.1 Copyright (c) 2002
//  Ziff Davis Media, Inc.
//  All rights reserved.
//  First Published in PC Magazine, US Edition, July 2000
//
//  Author : Mark Nelson
//  Date   : September 29, 2002
//
//  Description
//
//  This file contains the implementation for class Advanced Dialog. This
//  class is very generic - it simply allows the user to edit values that
//  are stored in CubeShow.ini. The only thing slightly out of the ordindary
//  is the manner in which it returns control to the caller. It passes a TRUE
//  or FALSE value to indicate whether the calling dialog needs to rescan the
//  image folder due to a change in the target BMP size.
//

#include "stdafx.h"
#include "cubeshow.h"
#include "AdvancedDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAdvancedDialog dialog


CAdvancedDialog::CAdvancedDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CAdvancedDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CAdvancedDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CAdvancedDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAdvancedDialog)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAdvancedDialog, CDialog)
	//{{AFX_MSG_MAP(CAdvancedDialog)
	ON_BN_CLICKED(IDC_HELP2, OnHelp2)
	ON_BN_CLICKED(IDC_RESTORE_DEFAULTS, OnRestoreDefaults)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAdvancedDialog message handlers

BOOL CAdvancedDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
    bool low = AfxGetApp()->GetProfileInt( "Settings", "LowScreenResolution", 0 ) ? true : false;
    SendDlgItemMessage( IDC_LOW_SCREEN_RESOLUTION, BM_SETCHECK, low ? BST_CHECKED : BST_UNCHECKED, 0 );
    
    bool allow_software_acceleration = AfxGetApp()->GetProfileInt( "Settings", "AllowSoftwareAcceleration", 0 ) ? true : false;
    SendDlgItemMessage( IDC_ALLOW_SOFTARE_ACCELERATION, BM_SETCHECK, allow_software_acceleration ? BST_CHECKED : BST_UNCHECKED, 0 );
    
    bool preserve = AfxGetApp()->GetProfileInt( "ConversionParms", "PreserveAspectRatio", 1 ) ? true : false;
    SendDlgItemMessage( IDC_PRESERVE_ASPECT_RATIO, BM_SETCHECK, preserve ? BST_CHECKED : BST_UNCHECKED, 0 );
	
    bool convert = AfxGetApp()->GetProfileInt( "Settings", "Convert", 1 ) ? true : false;
    SendDlgItemMessage( IDC_CONVERT, BM_SETCHECK, convert ? BST_CHECKED : BST_UNCHECKED, 0 );

	int texture_size = AfxGetApp()->GetProfileInt( "Settings", "TextureSize", 256 );
    SetDlgItemInt( IDC_TEXTURE_SIZE, texture_size, TRUE );
	
    int time_between_images = AfxGetApp()->GetProfileInt( "Settings", "TimeBetweenImages", 5000 );
    SetDlgItemInt( IDC_TIME_BETWEEN_IMAGES, time_between_images, TRUE );

    int target_bmp_size = AfxGetApp()->GetProfileInt( "ConversionParms", "TargetBmpSize", 256 );
    SetDlgItemInt( IDC_TARGET_BMP_SIZE, target_bmp_size, TRUE );

    //
    // Most of the controls in this dialog have tooltips. The tooltip control is created here,
    // and the messages for the various controls are initialized as well.
    // 
    m_tooltip.Create( this );
    m_tooltip.Activate( TRUE );
    m_tooltip.SetMaxTipWidth( 350 );

    m_tooltip.AddTool( GetDlgItem( IDC_LOW_SCREEN_RESOLUTION ), 
                       "When this setting is TRUE, CubeShow will drop down to a screen resolution"
                       " of 640 by 480 when activated. Your display will be returned to its normal"
                       " resolution upon exit." );

    m_tooltip.AddTool( GetDlgItem( IDC_ALLOW_SOFTARE_ACCELERATION ), 
                       "Setting this parameter to true will allow Direct3D to perform software"
                       " 3D acceleration. You may need to do this if your video card doesn\'t"
                       " support hardware acceleration." );

    char *msg1 = "This numeric parameter indicates the number of milliseconds between the load of"
                 " each new picture.";
    m_tooltip.AddTool( GetDlgItem( IDC_TIME_BETWEEN_IMAGES ), msg1 );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_1 ), msg1 );

    char *msg2 = "This numeric parameter indicates the number of milliseconds between the load of"
                 " each new picture.";
    m_tooltip.AddTool( GetDlgItem( IDC_TEXTURE_SIZE ), msg2 );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_2 ), msg2 );

    m_tooltip.AddTool( GetDlgItem( IDC_CONVERT ), 
                       "When this entry is selected, pictures added to CubeShow will " 
                       "be converted to BMP format and stored in the image cache directory" );

    m_tooltip.AddTool( GetDlgItem( IDC_PRESERVE_ASPECT_RATIO ), 
                       "When this entry is selected, pictures with an aspect ratio greater " 
                       "than 4:3 or less than 3:4 will get special handling when converted. "
                       "Instead of being stretched to fit a square cube face, black bands will "
                       " be inserted on the top and bottom or two sides of the image, ensuring "
                       " that no image distortion takes place" );

    char *msg3 = "This numeric parameter selects the size of the cached BMP file created during "
                 "the conversion process";
    m_tooltip.AddTool( GetDlgItem( IDC_TARGET_BMP_SIZE ), msg3 );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_3 ), msg3 );
    return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CAdvancedDialog::PreTranslateMessage(MSG* pMsg)
{
    {
        // Let the ToolTip process this message.
        m_tooltip.RelayEvent(pMsg);
    }
    return CDialog::PreTranslateMessage(pMsg);    // CG: This was added by the ToolTips component.
}

void CAdvancedDialog::OnCancel() 
{
	// TODO: Add extra cleanup here
	
//	CDialog::OnCancel();
    EndDialog( FALSE );
}

void CAdvancedDialog::OnOK() 
{
    int result = SendDlgItemMessage( IDC_LOW_SCREEN_RESOLUTION, BM_GETCHECK, 0, 0 );
    AfxGetApp()->WriteProfileInt( "Settings", "LowScreenResolution", result == BST_CHECKED ? 1 : 0 );

    int allow_software_acceleration = SendDlgItemMessage( IDC_ALLOW_SOFTARE_ACCELERATION, BM_GETCHECK, 0, 0 );
    AfxGetApp()->WriteProfileInt( "Settings", "AllowSoftwareAcceleration", allow_software_acceleration == BST_CHECKED ? 1 : 0 );

    int preserve = SendDlgItemMessage( IDC_PRESERVE_ASPECT_RATIO, BM_GETCHECK, 0, 0 );
    AfxGetApp()->WriteProfileInt( "ConversionParms", "PreserveAspectRatio", preserve == BST_CHECKED ? 1 : 0 );
    
    int convert = SendDlgItemMessage( IDC_CONVERT, BM_GETCHECK, 0, 0 );
    AfxGetApp()->WriteProfileInt( "Settings", "Convert", convert == BST_CHECKED ? 1 : 0 );

    int texture_size = GetDlgItemInt( IDC_TEXTURE_SIZE );
    AfxGetApp()->WriteProfileInt( "Settings", "TextureSize", texture_size );
	
    int time_between_images = GetDlgItemInt( IDC_TIME_BETWEEN_IMAGES );
    AfxGetApp()->WriteProfileInt( "Settings", "TimeBetweenImages", time_between_images );
    
    int target_bmp_size = GetDlgItemInt( IDC_TARGET_BMP_SIZE );
    int old_bmp_size = AfxGetApp()->GetProfileInt( "ConversionParms", "TargetBmpSize", 256 );

    AfxGetApp()->WriteProfileInt( "ConversionParms", "TargetBmpSize", target_bmp_size );

    //
    // If this dialog returns a TRUE from DoModal(), it means I need to resample all the images
    // because the user has selected a new target BMP size.
    //
    EndDialog( target_bmp_size != old_bmp_size );
}

void CAdvancedDialog::OnHelp2() 
{
    CString filename = ((CCubeShowApp *) AfxGetApp())->m_AppPath + "CubeShow.hlp";
    ::WinHelp( m_hWnd, filename, HELP_CONTEXT, 5 );	
	
}

void CAdvancedDialog::OnRestoreDefaults() 
{
    SendDlgItemMessage( IDC_LOW_SCREEN_RESOLUTION, BM_SETCHECK, BST_UNCHECKED, 0 );
    SendDlgItemMessage( IDC_ALLOW_SOFTARE_ACCELERATION, BM_SETCHECK, BST_UNCHECKED, 0 );
    SendDlgItemMessage( IDC_PRESERVE_ASPECT_RATIO, BM_SETCHECK, BST_CHECKED, 0 );
    SendDlgItemMessage( IDC_CONVERT, BM_SETCHECK, BST_CHECKED, 0 );
    SetDlgItemInt( IDC_TEXTURE_SIZE, 256, FALSE );
    SetDlgItemInt( IDC_TIME_BETWEEN_IMAGES, 5000, FALSE );
    SetDlgItemInt( IDC_TARGET_BMP_SIZE, 256, FALSE );
}
