//
//   Convert.cpp : Implementation file for class CConvert
//
//  CubeShow Version 1.0 Copyright (c) 2002
//
//  Ziff Davis Media, Inc.
//  All rights reserved.
//  First Published in PC Magazine, US Edition, July 2000
//
//  Sample code in CreateBitmapInfoStruct() and CreateBMPFile() 
//  is Copyright 2002 Microsoft Corporation.
//
//
//  Author : Mark Nelson
//  Date   : May 23, 2002
//
//  Description
//
//  This class is responsible for performing the conversion/downsizing
//  step when the user selects a new directory of image files. Each file is
//  read into a DC, resized to be around 256x256, and saved to a BMP file in
//  the Image Cache folder under the app folder. The hard work of loading
//  the image is done by the OleLoadPicture function, which loads a JPG
//  into an IPicture interface. Writing the BMP file is done using sample
//  code MSDN code from Microsoft.
//

#include "stdafx.h"
#include "cubeshow.h"
#include "Convert.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CConvert dialog

CConvert::CConvert(CWnd* pParent /*=NULL*/)
: CDialog(CConvert::IDD, pParent)
{
    //{{AFX_DATA_INIT(CConvert)
        // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_KillThread = false;
    m_bConvert = true;
    m_TargetBmpSize = 256;
}

CConvert::~CConvert()
{
}

void CConvert::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CConvert)
    DDX_Control(pDX, IDCANCEL, m_CancelButton);
    DDX_Control(pDX, IDC_ABORT, m_AbortButton);
    DDX_Control(pDX, IDC_PROGRESS, m_Progress);
    DDX_Control(pDX, IDOK, m_OkButton);
    //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CConvert, CDialog)
    //{{AFX_MSG_MAP(CConvert)
    ON_BN_CLICKED(IDC_ABORT, OnAbort)
    ON_WM_SETCURSOR()
    ON_MESSAGE( WM_USER, OnUser )
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConvert message handlers

//
// All of the file conversions take place in a worker thread that is created
// here. The worker thread updates the m_Progress slider as it churns through
// the list of files. The thread can either die a natural death when it
// completes the list of files, or it can be killed by the end user hitting
// the Abort key, which sets the m_KillThread member.
//
BOOL CConvert::OnInitDialog() 
{
    CDialog::OnInitDialog();
        
    m_Progress.SetRange( 0, 100 );
    m_Progress.SetPos( 0 );
    converterThread = AfxBeginThread( Converter, this );
    m_hConverterThread = INVALID_HANDLE_VALUE;
    HRESULT hr =  ::DuplicateHandle( GetCurrentProcess(),         // source process
                                     converterThread->m_hThread,  // original handle
                                     GetCurrentProcess(),         // destination process
                                     &m_hConverterThread,         // new duplicate handle
                                     0,                           // access rights (overridden by last parameter) 
                                     FALSE,                       // children do not inherit the handle 
                                     DUPLICATE_SAME_ACCESS );     // copy access rights from original handle

    return TRUE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
}

//
// The user of CubeShow has to select a single folder as the source of all
// image files. When the config dialog instantiates this CConvert object,
// it calls this method before the dialog is shown. This method does a bit 
// more than actually just setting the source name. It also loads up the
// m_InputFiles list member with a list of all the file names it found in
// that directory.
//
// Note that the GIF patent expires at the end of 2002.
//
void CConvert::SetSourcePath(CString path)
{
    //
    // Add the "*.*" search template to the path.
    //
    if ( path.Right( 1 ) != "/" && path.Right( 1 ) != "\\" )
        path += "\\";
    path += "*.*";
    //
    // From this point on the routine is a very conventional
    // CFileFind loop.
    //
    CFileFind finder;
    BOOL bWorking = finder.FindFile( path );
    while ( bWorking )
    {
        bWorking = finder.FindNextFile();
        CString name = finder.GetFilePath();
        int dot = name.ReverseFind( '.' );
        CString extension;
        if ( dot != -1 )
            extension = name.Right( name.GetLength() - dot - 1 );
        else
            extension = "";
        extension.MakeUpper();
        if ( extension == "BMP" || extension == "JPG" || extension == "JPEG" || extension == "DIB" || extension == "WMF" )
            m_InputFiles.push_back( name );
    }
}

//
// After the dialog shuts down, the Config Dialog gets the number
// of processed files so it can pop up a notification message box
// to the user.
//
int CConvert::GetProcessedCount()
{
   return m_OutputFiles.size();
}

//
// The config dialog uses this method when it is deciding whether or
// not to do a conversion. After calling SetSourcePath(), this routine
// will return a valid count of the number of images in this directory.
// If that number doesn't match the number currently stored in the
// INI file, a conversion will definitely take place.
//
int CConvert::GetInputCount()
{
   return m_InputFiles.size();
}

//
// If the user hits the abort key, the main dialog pops up
// the hourglass cursor to indicate that the conversion thread
// is going to be aborted. When that thread finally aborts, it
// sends a WM_USER message to the dialog window, which goes to
// the message handler. Here the cursor is changed back to the
// standard cursor.
// 
void CConvert::OnUser(WPARAM, LPARAM)
{
    HICON h = ::AfxGetApp()->LoadStandardCursor( IDC_ARROW );
    SetCursor( h );
}

//
// This static method of CConvert is the worker thread that
// actually performs the conversions. The process is fairly
// simple. The thread gets the name of the next input file
// from the m_InputFiles member. It then creates the name of
// the output file, which is the input file renamed to type
// BMP and moved to the image cache folder. Once that is done,
// the Convert() routine is called to perform the actual
// conversion.
//
// Tricky bits in here of note:
//
// First, it is possible to have multiple input files with
// the same name but a different extension. The routine that
// creates the output file name has to check to see if the
// output file name already exists. If it does, a bit of
// mangling takes place to create a new name that hasn't been
// used already.
//
// The m_KillThread member is checked before every conversion
// to see if it's time to die.
//
// m_OutputFiles gets a copy of the name of each file as it is
// converted. This output file name will be used by the config
// dialog to add all entries to the INI file.
//
// If the conversion fails, we add the name of the input file
// directly to the output file queue. This means it will be loaded
// directly as a texture. If it's a really big file this might
// take a bit longer than desired, but this should be preferable
// to just throwing it out.
//
// When the thread exits, the user is ready to move on, which means
// the OK button is enabled, the Cancel button is enabled, and just
// in case, the cursor is set to be a normal arrow.
//
// If the Convert flag in the INI file is set to 0, the conversion
// routine is never actually called. In that case, the input file
// name is copied directly to the output queue.
//
// If the thread is exiting under normal conditions, it simulates a
// button press of the "OK" button, which will close the dialog and
// return to the config dialog.
//
UINT CConvert::Converter(LPVOID lParam)
{
    CConvert *dlg = (CConvert *) lParam;
    int total = dlg->m_InputFiles.size();
    int count = 0;
    while ( dlg->m_InputFiles.size() > 0 )
    {
        if ( dlg->m_KillThread )
        {
            dlg->SetDlgItemText( IDC_MESSAGE, "Aborted at User Request" );
            break;
        }
        //
        // The next long block of code creates a valid output file name.
        // This is long enough that it should probably be moved to a
        // separate routine.
        //
        CString file = dlg->m_InputFiles.front();
        CString name = file;
        int spot = name.ReverseFind( '\\' );
        if ( spot == -1 )
            spot = name.ReverseFind( ':' );
        if ( spot != -1 )
            name = name.Mid( spot + 1 );
        dlg->m_InputFiles.pop_front();
        CString outputFile;
        for ( int i = -1 ; i < 99 ; i++ )
        {
            outputFile = ((CCubeShowApp *) AfxGetApp())->m_AppPath + "ImageCache\\" + name;
            spot = outputFile.ReverseFind( '.' );
            outputFile = outputFile.Left( spot );
            if ( i == -1 )
            {
                outputFile += ".bmp";
            } else {
                char buf[ _MAX_PATH ];
                sprintf( buf, "%s-%d.bmp", outputFile, i );
                outputFile = buf;
            }
            int result = _access( outputFile, 0 );
            if (  result != 0 )
                break;
        }
        //
        // Finally! I have an input file name, an output file name, and I can
        // invoke the conversion routine.
        //
        if ( !dlg->m_bConvert )
            dlg->SetDlgItemText( IDC_MESSAGE, "Indexing " + name );
        else
        {
            CString msg;
            msg.Format( "Input: %s\r\nOutput: %s", name, CString( "ImageCache\\" ) + name );
            dlg->SetDlgItemText( IDC_MESSAGE, msg );
        }
        if ( dlg->Convert( file, outputFile ) && dlg->m_bConvert  )
            dlg->m_OutputFiles.push_back( outputFile );
        else
            dlg->m_OutputFiles.push_back( file );
        dlg->m_Progress.SetPos( ++count * 100 / total );
    }
    //
    // This is the only way out of the thread. Do some cleanup first.
    //
    dlg->SendMessage( WM_USER, 0, 0 );
    dlg->m_OkButton.EnableWindow();
    dlg->m_CancelButton.EnableWindow();
    if ( !dlg->m_KillThread )
        ::PostMessage( dlg->GetDlgItem( IDOK )->m_hWnd, BM_CLICK, 0, 0 );
    return 0;
}

//
// This handler deals with the user pressing the Abort button. Doing so sets
// the kill flag and disables the abort button. The conversion thread should
// very quickly see that the flag is set. It will then exit, which enables
// the OK button and lets the user out.
//
void CConvert::OnAbort() 
{
    m_KillThread = true;
    m_AbortButton.EnableWindow( FALSE );
    ::SetCursor( ::AfxGetApp()->LoadStandardCursor( IDC_WAIT ) );
    SetDlgItemText( IDC_MESSAGE, "Aborting build of image folder" );
}

//
// This dialog starts with the OK button disabled. The only way we
// actually see this button pressed is either 1) The conversion thread
// completes on its own, in which case it sends a button press to the
// dialog window, or 2) The user presses OK after it is enabled from
// an abort request.
//
// Just to be sure everything is kosher, this guy waits until the
// conversion thread is really dead before exiting.
//
void CConvert::OnOK() 
{
    HICON h = ::AfxGetApp()->LoadStandardCursor( IDC_ARROW );
    DWORD result = ::WaitForSingleObject( m_hConverterThread, 10000 );        
    if ( result != WAIT_OBJECT_0 )
    {
        DWORD dw = ::GetLastError();
        MessageBox( "An error occurred trying to kill the processing thread" );
        ::TerminateThread( m_hConverterThread, 0 );
    }
    CloseHandle( m_hConverterThread );
    CDialog::OnOK();
}

//
// In order for this dialog to manipulate its cursor, I have to
// handle this message. By returning TRUE I indicate that yes,
// this is really the cursor I want to see.
//

BOOL CConvert::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
  return TRUE;
}

//
// After the conversion is complete, either by finishing all the files
// or by hitting the abort button, the user can exit this dialog by
// pressing "OK" or "Cancel". If the user presses Cancel, that tells
// the config dialog that it shouldn't use the new list of files, the
// user has chosen to discard them.
//
void CConvert::OnCancel() 
{
    HICON h = ::AfxGetApp()->LoadStandardCursor( IDC_ARROW );
    DWORD result = ::WaitForSingleObject( m_hConverterThread, 10000 );        
    if ( result != WAIT_OBJECT_0 )
    {
        DWORD dw = ::GetLastError();
        MessageBox( "An error occurred trying to kill the processing thread" );
        ::TerminateThread( m_hConverterThread, 0 );
    }
    CloseHandle( m_hConverterThread );
    ::SetCursor( ::AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );
        CDialog::OnCancel();
}


//
// The IPicture interface is used to load a picture into a DC. Once it is there, the
// process of writing the data to a BMP file is performed by two routines Microsoft
// provides to illustrate the procedure:
//
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdi/bitmaps_7zfp.asp
//
// The only changes between this code and Microsoft's is the removal of their error 
// handling code.
//

PBITMAPINFO CreateBitmapInfoStruct(HBITMAP hBmp )
{ 
    BITMAP bmp; 
    PBITMAPINFO pbmi; 
    WORD    cClrBits; 

    // Retrieve the bitmap color format, width, and height. 
    if (!GetObject(hBmp, sizeof(BITMAP), (LPSTR)&bmp)) 
        return 0;

    // Convert the color format to a count of bits. 
    cClrBits = (WORD)(bmp.bmPlanes * bmp.bmBitsPixel); 
    if (cClrBits == 1) 
        cClrBits = 1; 
    else if (cClrBits <= 4) 
        cClrBits = 4; 
    else if (cClrBits <= 8) 
        cClrBits = 8; 
    else if (cClrBits <= 16) 
        cClrBits = 16; 
    else if (cClrBits <= 24) 
        cClrBits = 24; 
    else cClrBits = 32; 

    // Allocate memory for the BITMAPINFO structure. (This structure 
    // contains a BITMAPINFOHEADER structure and an array of RGBQUAD 
    // data structures.) 

     if (cClrBits != 24) 
         pbmi = (PBITMAPINFO) LocalAlloc(LPTR, 
                    sizeof(BITMAPINFOHEADER) + 
                    sizeof(RGBQUAD) * (1<< cClrBits)); 

     // There is no RGBQUAD array for the 24-bit-per-pixel format. 

     else 
         pbmi = (PBITMAPINFO) LocalAlloc(LPTR, 
                    sizeof(BITMAPINFOHEADER)); 

    // Initialize the fields in the BITMAPINFO structure. 

    pbmi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER); 
    pbmi->bmiHeader.biWidth = bmp.bmWidth; 
    pbmi->bmiHeader.biHeight = bmp.bmHeight; 
    pbmi->bmiHeader.biPlanes = bmp.bmPlanes; 
    pbmi->bmiHeader.biBitCount = bmp.bmBitsPixel; 
    if (cClrBits < 24) 
        pbmi->bmiHeader.biClrUsed = (1<<cClrBits); 

    // If the bitmap is not compressed, set the BI_RGB flag. 
    pbmi->bmiHeader.biCompression = BI_RGB; 

    // Compute the number of bytes in the array of color 
    // indices and store the result in biSizeImage. 
    // For Windows NT, the width must be DWORD aligned unless 
    // the bitmap is RLE compressed. This example shows this. 
    // For Windows 95/98/Me, the width must be WORD aligned unless the 
    // bitmap is RLE compressed.
    pbmi->bmiHeader.biSizeImage = ((pbmi->bmiHeader.biWidth * cClrBits +31) & ~31) /8
                                  * pbmi->bmiHeader.biHeight; 
    // Set biClrImportant to 0, indicating that all of the 
    // device colors are important. 
    pbmi->bmiHeader.biClrImportant = 0; 
    return pbmi; 
} 

//
// See the comments for the previous routine. This is the second piece of sample
// code Microsoft uses to illustrate the steps needed to create a BMP file.
//
bool CreateBMPFile( const char* pszFile, PBITMAPINFO pbi, HBITMAP hBMP, HDC hDC )
{ 
    HANDLE hf;                  // file handle 
    BITMAPFILEHEADER hdr;       // bitmap file-header 
    PBITMAPINFOHEADER pbih;     // bitmap info-header 
    LPBYTE lpBits;              // memory pointer 
    DWORD dwTotal;              // total count of bytes 
    DWORD cb;                   // incremental count of bytes 
    BYTE *hp;                   // byte pointer 
    DWORD dwTmp; 

    pbih = (PBITMAPINFOHEADER) pbi; 
    lpBits = (LPBYTE) GlobalAlloc(GMEM_FIXED, pbih->biSizeImage);

    if (!lpBits) 
         return false;

    // Retrieve the color table (RGBQUAD array) and the bits 
    // (array of palette indices) from the DIB. 
    if (!GetDIBits(hDC, hBMP, 0, (WORD) pbih->biHeight, lpBits, pbi, 
        DIB_RGB_COLORS)) 
    {
        return false;
    }

    // Create the .BMP file. 
    hf = CreateFile( pszFile, 
                     GENERIC_READ | GENERIC_WRITE, 
                     (DWORD) 0, 
                     NULL, 
                     CREATE_ALWAYS, 
                     FILE_ATTRIBUTE_NORMAL, 
                     (HANDLE) NULL); 
    if ( hf == INVALID_HANDLE_VALUE ) 
        return false;
    hdr.bfType = 0x4d42;        // 0x42 = "B" 0x4d = "M" 
    // Compute the size of the entire file. 
    hdr.bfSize = (DWORD) (sizeof(BITMAPFILEHEADER) + 
                 pbih->biSize + pbih->biClrUsed 
                 * sizeof(RGBQUAD) + pbih->biSizeImage); 
    hdr.bfReserved1 = 0; 
    hdr.bfReserved2 = 0; 

    // Compute the offset to the array of color indices. 
    hdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) + 
                    pbih->biSize + pbih->biClrUsed 
                    * sizeof (RGBQUAD); 

    // Copy the BITMAPFILEHEADER into the .BMP file. 
    if (!WriteFile(hf, (LPVOID) &hdr, sizeof(BITMAPFILEHEADER), 
        (LPDWORD) &dwTmp,  NULL)) 
    {
       return false;
    }

    // Copy the BITMAPINFOHEADER and RGBQUAD array into the file. 
    if (!WriteFile(hf, (LPVOID) pbih, sizeof(BITMAPINFOHEADER) 
                  + pbih->biClrUsed * sizeof (RGBQUAD), 
                  (LPDWORD) &dwTmp, ( NULL)))
       return false;

    // Copy the array of color indices into the .BMP file. 
    dwTotal = cb = pbih->biSizeImage; 
    hp = lpBits; 
    if (!WriteFile(hf, (LPSTR) hp, (int) cb, (LPDWORD) &dwTmp,NULL)) 
           return false;

    // Close the .BMP file. 
     if (!CloseHandle(hf)) 
           return false;

    // Free memory. 
    GlobalFree((HGLOBAL)lpBits);
    return true;
}

//
// This is the conversion routine where all the actual work gets done.
// Step one is load the picture into memory,which is done via the
// IPicture interface. Step two is to render it to a DC, which is
// complicated by possible attempts to preserve aspect ratio. The
// final step is to write the resulting file out to a BMP file. The 
// routine returns a boolean to the worker thread, telling it whether
// or not to use the converted file name or the original file name
// in the INI file.
//
bool CConvert::Convert( CString &inputFile, CString &outputFile )
{
    if ( !m_bConvert )
    {
        Sleep( 200 ); // A little delay so the user can see the names being converted
        return false;
    }
    bool bResult = false;
    //
    // OleLoadPicture is used to load the picture into an IPicture interface. It takes
    // care of the really difficult work of interpreting the image file and converting
    // it to data in memory.
    //
    IPicture *picture = NULL;
    ifstream file( inputFile, ios_base::binary | ios_base::in );
    if ( file )
    {
        //
        // I need to create an IStream for the call to OleLoadPicture, it doesn't 
        // read files. I accomplish this by reading the file into a Windows memory
        // buffer and converting that to a stream.
        //
        file.seekg( 0, ios_base::end );
        long length = file.tellg();
        file.seekg( 0, ios_base::beg );

        HGLOBAL hGlobal = GlobalAlloc( GMEM_MOVEABLE, length );
        void* p = GlobalLock(hGlobal);
        file.read( (char *) p, length );
        GlobalUnlock( hGlobal );
        IStream* pStream = NULL;
        if ( CreateStreamOnHGlobal( hGlobal, TRUE, &pStream ) == S_OK )
        {
            HRESULT hr = ::OleLoadPicture( pStream, 
                                           length, 
                                           FALSE, 
                                           IID_IPicture, 
                                           (LPVOID *) &picture );
            pStream->Release();
            //
            // At this point I have a valid IPicture. I need to render its data to
            // a Bitmap in memory. The bitmap is going to be 256X256 (unless overridden
            // in the INI file.) The only thing I have to decide is whether this aspect
            // ratio is far enough off from 1:1 that it needs to be preserved. If so, it
            // gets this slightly special handling.
            //
            if ( hr == S_OK )
            {
                CDC dc;
                dc.CreateCompatibleDC( NULL );
                CBitmap bm;
                OLE_XSIZE_HIMETRIC width;
                OLE_YSIZE_HIMETRIC height;
                picture->get_Width( &width );
                picture->get_Height( &height );
                CSize sz( width, height );
                dc.HIMETRICtoDP( &sz );
                int bmWidth;
                int bmHeight;
                int base = m_TargetBmpSize;
                float ratio = (float) width / float( height );
                if ( ratio > 1.35 || ratio < .73 )
                {
                    Trace( "Preserving aspect ratio for image %s\n", inputFile );
                    hr = bm.CreateCompatibleBitmap( GetDC(), base, base );
                    void *old = dc.SelectObject( bm );
                    CRect rect( 0, 0, base, base );
                    CBrush brush;
                    brush.CreateSolidBrush( 0x000000 );
                    dc.FillRect( rect, &brush );
                    int bmX;
                    int bmY;
                    if ( ratio > 1.0 )
                    {
                        bmWidth = base;
                        bmHeight = base / ratio;
                        bmX = 0.0;
                        bmY = ( base - bmHeight ) / 2;
                    } else {
                        bmHeight = base;
                        bmWidth = base * ratio;
                        bmX = ( base - bmWidth ) / 2;
                        bmY = 0.0;
                    }
                    hr = picture->Render( dc,
                                          bmX, bmY,
                                          bmWidth, bmHeight,
                                          0, height,
                                          width, -height,
                                          NULL );
                    CBrush brush2;
                    brush2.CreateSolidBrush( 0x404040 ); //draw a single pixel border
                    CRect rect2( 0, 0, base, base );
                    dc.FrameRect( rect2, &brush2 );
                } else {  //aspect ratio within range, expand picture to fill the cube
                    if ( sz.cx > sz.cy )
                    {
                        bmWidth = base;
                        bmHeight = (double) base * (double) sz.cy / (double) sz.cx;
                    } else
                    {
                        bmWidth = (double) base * (double) sz.cx / (double) sz.cy;
                        bmHeight = base;
                    }
                    hr = bm.CreateCompatibleBitmap( GetDC(), bmWidth, bmHeight );
                    void *old = dc.SelectObject( bm );
                    hr = picture->Render( dc,
                                          0, 0,
                                          bmWidth, bmHeight,
                                          0, height,
                                          width, -height,
                                          NULL );
                }
                //
                // At this point I have the picture stored in the bitmap. I just need 
                // to call Microsoft's code to store it in the BMP file and all is well.
                //
                PBITMAPINFO pbmi = CreateBitmapInfoStruct( (HBITMAP) bm );
                CreateBMPFile( outputFile, pbmi, (HBITMAP) bm, dc );
                LocalFree( pbmi );
                picture->Release();

                return true;
            } 
            if ( picture != NULL) //OleLoadPicture returned an error
                picture->Release();
            return false;
        } 
        return false;  //Failed to create IStream interface
    }
    else  //Unable to open the input file
        return false;
}

//
// Rather than read the Convert option from the INI file before translating every file, 
// this class relies on the config dialog to do it once and store the value using this
// method
//
void CConvert::SetConvert( bool convert )
{
  m_bConvert = convert;
}
