//
//   CubeShow.cpp : Implementation file for class CCubeShowApp
//
//  CubeShow Version 1.0 Copyright (c) 2002
//  Ziff Davis Media, Inc.
//  All rights reserved.
//  First Published in PC Magazine, US Edition, July 2000
//
//  Author : Mark Nelson
//  Date   : May 23, 2002
//
//  Description
//
//  This class is the application class derived from CWinApp, and has
//  the MFC-defined responsibilities for starting the app, parsing the
//  command line, and creating the windows.
//
//  Much of the code in this source file was posted on CodeGuru by chensu
//  at the following URL: http://www.codeguru.com/misc/scrfrmwk.shtml.
//  CodeGuru allows unrestricted use of the code samples on their web
//  site, and is well worth a visit at http://www.codeguru.com.
//

#include "stdafx.h"
#include "CubeShow.h"
#include "drawwnd.h"
#include "CubeShowWnd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

BEGIN_MESSAGE_MAP(CCubeShowApp, CWinApp)
    //{{AFX_MSG_MAP(CCubeShowApp)
    //}}AFX_MSG
    ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

CCubeShowApp::CCubeShowApp()
{
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CCubeShowApp object

CCubeShowApp theApp;

//
// In the world of MFC, InitInstance() is where all significant initialization takes place.
// Much of the code in here is boilerplate that is created by the MFC AppWizard.
//
BOOL CCubeShowApp::InitInstance()
{
    CreateMutex( NULL, FALSE, "CubeShowRunning");
    this->Enable3dControls();
    //
    // All of CubeShow's initialization information is stored in CubeShow.ini, which is in the 
    // app directory. The only way to find the app directory is to look in the registry to see
    // where InnoSetup placed most of the app files. Once I have the app folder, I can set the
    // m_szProfileName member, and all calls to the MFC INI functions will automatically route
    // through to the proper file.
    //
    CRegKey key;
    LONG result = key.Open( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\CubeShow_is1", KEY_READ );
    char appPath[ _MAX_PATH ];
    DWORD length = _MAX_PATH;
    if ( result != ERROR_SUCCESS || ERROR_SUCCESS != key.QueryValue( appPath, "Inno Setup: App Path", &length ) )
    {
        Trace( "Couldn't find app path in registry... Aborting\n" );
        MessageBox( NULL, "CubeShow's registry information is missing. Did you use the CubeShow install program?", "CubeShow Fatal Error", MB_ICONWARNING | MB_OK | MB_SYSTEMMODAL );
        PostQuitMessage( 0 );
    }
    m_AppPath = appPath;
    if ( m_AppPath.Right( 1 ).Compare( "\\" ) != 0 )
        m_AppPath += "\\";
    m_pszProfileName = _tcsdup( m_AppPath + "CubeShow.ini" );

    //
    // The next job of InitInstance() is to parse the command line and create the appropriate window
    // based on that. Member GetScrMode() parses the command line and optionally returns a window
    // handle that was passed on the command line. The command line will be one of the standard ones
    // defined for Win32 screen savers.
    //
    HWND hWnd;
    switch (this->GetScrMode(hWnd))
    {
        case SMSAVER:
        {
            // 
            // Run the screen saver in full screen mode. No parent window, CCubeShowWnd is created
            // and takes over the entire screen.
            //
            Trace( "*\n" );
            Trace( "* CubeShow starting in screen saver mode\n" );
            Trace( "*\n" );
            CCubeShowWnd *pWnd = new CCubeShowWnd;
            pWnd->Create();
            m_pMainWnd = pWnd;
            return TRUE;
        }
        case SMTEST:
        {
            //
            // Run the screen saver in test mode. CCubeShowWnd has an alternate CreateTest() method
            // which creates a resizable window instead of starting in full screen mode.
            //
            Trace( "*\n" );
            Trace( " CubeShow starting in test mode\n" );
            Trace( "*\n" );
            Trace( "Starting in test mode\n" );
            CCubeShowWnd *pWnd = new CCubeShowWnd;
            pWnd->CreateTest();
            m_pMainWnd = pWnd;
            return TRUE;
        }
        
        case SMPREVIEW:
        {
            //
            // Thumbnail preview mode. The HWND value passed on the command line is the 
            // handle for the window that the screen saver should render to. In the previous
            // two modes, CCubeShowWnd provided the window that was the target of the
            // screen saver. In this case, CDrawWnd is given a handle of a window from some
            // other process (usually the Display Properties dialog.)
            //
            Trace( "*\n" );
            Trace( "* CubeShow starting in thumbnail preview mode\n" );
            Trace( "*\n" );
            CWnd *pWndParent = CWnd::FromHandle(hWnd);
            ASSERT(pWndParent != NULL);
            CDrawWnd *pWnd = new CDrawWnd();
            CRect rect;
            pWndParent->GetClientRect(&rect);
            pWnd->Create( 0, 
                          "CubeShow Preview",
                          WS_VISIBLE | WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_OVERLAPPED, 
                          rect, 
                          CWnd::FromHandle(hWnd) );
            m_pMainWnd = pWnd;
            return TRUE;
        }
        
        case SMCONFIG:
        {
            // 
            // A completely different way to start the program, this brings up the configuration
            // dialog instead of a screen saver.
            //
            Trace( "*\n" );
            Trace( " CubeShow starting in config mode\n" );
            Trace( "*\n" );
            this->DoConfig( hWnd );
            return FALSE;
        }
        case SMPASSWORD:
        {
            //
            // Under Windows 9X, the screen saver can be asked to change the user's password, which
            // I do by calling this routine.
            //
            Trace( "*\n" );
            Trace( " CubeShow starting in password entry mode\n" );
            Trace( "*\n" );
            this->DoChangePassword( hWnd );
            return FALSE;
        }
    }
    return FALSE;
}

//
// This routine parses the command line and returns the appropriate
// mode back to the InitInstance() routine. It has just two things to do. 
// 1) Scan for the command line option, which is one of /C, /S, /T, /P, or /A. 
// 2) Get the window handle off the command line if it is one of the commands
// that has a window handle.
//
// This code is unchanged from the original CodeGuru version and is used
// with permission.
//
CCubeShowApp::SCRMODE CCubeShowApp::GetScrMode(HWND &hWnd) const
{
    SCRMODE ScrMode = SMNONE;
    hWnd = NULL;

    TCHAR *pc = ::GetCommandLine();

    if (*pc == _T('\"'))
    {
        pc++;
        while (*pc != _T('\0') && *pc != _T('\"'))
            pc++;
    }
    else
    {
        while (*pc != _T('\0') && *pc != _T(' '))
            pc++;
    }

    if (*pc != _T('\0'))
        pc++;

    while (*pc == _T(' '))
        pc++;

    if (*pc == _T('\0'))
    {
        ScrMode = SMCONFIG;
        hWnd = NULL;
    }
    else
    {
        if (*pc == _T('-') || *pc == _T('/'))
            pc++;

        if (*pc == _T('s') || *pc == _T('S'))
        {
            ScrMode = SMSAVER;
        }
        else
            if (*pc == _T('p') || *pc == _T('P') ||
                *pc == _T('l') || *pc == _T('L'))
            {
                pc++;
                while (*pc == _T(' ') || *pc == _T(':'))
                    pc++;

                ScrMode = SMPREVIEW;
                hWnd = reinterpret_cast<HWND>(::_ttoi(pc));
            }
            else
                if (*pc == _T('c') || *pc == _T('C'))
                {
                    pc++;
                    while (*pc == _T(' ') || *pc == _T(':'))
                        pc++;
                    
                    ScrMode = SMCONFIG;
//
// If I'm trying to test config mode from the debugger, I don't want 
// to have a parent window. There were some nasty potential lockups
// that can be caused by virtue of the fact that the config dialog
// is modal - it could lock out access to the debugger.
//
#ifdef _DEBUG
                    hWnd = NULL;
#else
                    hWnd = *pc == _T('\0')
                           ? ::GetForegroundWindow()
                           : reinterpret_cast<HWND>(::_ttoi(pc));
#endif
                }
                else
                    if (*pc == _T('a') || *pc == _T('A'))
                    {
                        pc++;
                        while (*pc == _T(' ') || *pc == _T(':'))
                            pc++;
                        
                        ScrMode = SMPASSWORD;
                        hWnd = reinterpret_cast<HWND>(::_ttoi(pc));
                    }
                else
                    if (*pc == _T('t') || *pc == _T('T'))
                    {
                        ScrMode = SMTEST;
                    }
    }

    return ScrMode;
}

//
// All of the values in the config dialog are loaded up from
// CubeShow.ini before popping it up. If the user exits the
// dialog by way of the OK button, all the new values are
// written out to the INI file. There is one set of INI entries
// that aren't written here. The Image count and the names of
// the image files are actually written in the config dialog
// itself.
//
void CCubeShowApp::DoConfig(HWND hWndParent) 
{
    CCubeShowConfigDialog *dlg;
    if ( hWndParent != NULL )
        dlg = new CCubeShowConfigDialog( CWnd::FromHandle( hWndParent ) );
    else
        dlg = new CCubeShowConfigDialog();

    dlg->m_SliderSpinX = GetProfileInt( "Settings", "SpinX", 40 );
    dlg->m_SliderSpinY = GetProfileInt( "Settings", "SpinY", 60 );
    dlg->m_SliderSpinZ = GetProfileInt( "Settings", "SpinZ", 0  );
    dlg->m_SliderSpeedX = GetProfileInt( "Settings", "SpeedX", 60 );
    dlg->m_SliderSpeedY = GetProfileInt( "Settings", "SpeedY", 20  );
    dlg->m_SliderSpeedZ = GetProfileInt( "Settings", "SpeedZ", 0  );
    dlg->m_SliderCubeSize = GetProfileInt( "Settings", "CubeSize", 0  );
    dlg->m_KeepUpright = GetProfileInt( "Settings", "KeepUpright", 0 );
    dlg->m_PicturePath = GetProfileString( "Settings", "PicturePath" );
    int preserve = GetProfileInt( "ConversionParms", "PreserveAspectRatio", 1 );
    CString oldPathValue = dlg->m_PicturePath;
    if ( dlg->DoModal() == IDOK )
    {
        WriteProfileInt( "Settings", "SpinX", dlg->m_SliderSpinX );
        WriteProfileInt( "Settings", "SpinY", dlg->m_SliderSpinY );
        WriteProfileInt( "Settings", "SpinZ", dlg->m_SliderSpinZ );
        WriteProfileInt( "Settings", "SpeedX", dlg->m_SliderSpeedX );
        WriteProfileInt( "Settings", "SpeedY", dlg->m_SliderSpeedY );
        WriteProfileInt( "Settings", "SpeedZ", dlg->m_SliderSpeedZ );
        WriteProfileInt( "Settings", "CubeSize", dlg->m_SliderCubeSize );
        WriteProfileInt( "Settings", "KeepUpright", dlg->m_KeepUpright );
        WriteProfileString( "Settings", "PicturePath", dlg->m_PicturePath );
        WriteProfileInt( "ConversionParms", "PreserveAspectRatio", preserve );
    }
    delete dlg;
}

//
// This routine is also unchanged from the CodeGuru source. Under Windows 9X the
// Screen Saver can be asked to do let the user change the password. Microsoft
// recommends finding a pointer to PwdChangePasswordA in MPR.DLL and calling that. 
// So that's exactly what we do here.
//
void CCubeShowApp::DoChangePassword(HWND hWndParent) const
{
    // This only gets called on Windows 9x, when started with the /a option.
    HINSTANCE hmpr = ::LoadLibrary(_T("MPR.DLL"));

    if (hmpr != NULL)
    {
        typedef VOID (WINAPI *PPWDCHANGEPASSWORD)( LPCSTR lpcRegkeyname, 
                                                   HWND hWnd,
                                                   UINT uiReserved1, 
                                                   UINT uiReserved2 );
        PPWDCHANGEPASSWORD pPwdChangePassword =
            reinterpret_cast<PPWDCHANGEPASSWORD>
                (::GetProcAddress(hmpr, "PwdChangePasswordA"));

        if ( pPwdChangePassword != NULL )
            pPwdChangePassword( "SCRSAVE", hWndParent, 0, 0 );
        else
            TRACE( _T( "PwdChangeProc not found: cannot change password.\n" ) ); 
        ::FreeLibrary(hmpr);
    }
    else
        TRACE(_T("MPR.DLL not found: cannot change password.\n"));
}
