//
//   CubeShowConfigDialog.cpp : Implementation file for class CCubeShowConfigDialog
//
//  CubeShow Version 1.0 Copyright (c) 2002
//  Ziff Davis Media, Inc.
//  All rights reserved.
//  First Published in PC Magazine, US Edition, July 2000
//
//  Author : Mark Nelson
//  Date   : May 23, 2002
//
//  Description
//
//  This class is derived from CDialog, and does all the work required to present
//  the user the configuration dialog. It is normally presented to the user after
//  pressing the Settings button in the Display Properties dialog when on the Screen
//  Saver tab. All the options displayed in this dialog reflect settings seen in
//  CubeShow.ini. However, this dialog relies on whoever invokes it to set up all
//  the values that are read from the INI file, and expects the same caller to store
//  all the new values. One exception: the list of image files and their count are
//  managed directly here.
//

#include "stdafx.h"
#include "cubeshow.h"
#include "CubeShowConfigDialog.h"
#include "Convert.h"
#include "AboutBox.h"
#include "AdvancedDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCubeShowConfigDialog dialog


CCubeShowConfigDialog::CCubeShowConfigDialog(CWnd* pParent /*=NULL*/)
    : CDialog(CCubeShowConfigDialog::IDD, pParent)
{
    //{{AFX_DATA_INIT(CCubeShowConfigDialog)
        // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_ForceConvert = false;
}


void CCubeShowConfigDialog::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CCubeShowConfigDialog)
    DDX_Control(pDX, IDOK, m_OkButton);
    //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCubeShowConfigDialog, CDialog)
    //{{AFX_MSG_MAP(CCubeShowConfigDialog)
    ON_WM_DESTROY()
    ON_BN_CLICKED(IDC_SELECT_DIRECTORY_BUTTON, OnSelectDirectoryButton)
    ON_BN_CLICKED(ID_ABOUT, OnAbout)
    ON_EN_CHANGE(IDC_PICTURE_PATH, OnChangePicturePath)
    ON_BN_CLICKED(ID_HELP, OnHelp)
	ON_BN_CLICKED(IDC_ADVANCED, OnAdvanced)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCubeShowConfigDialog message handlers

//
// The function is used to find the full path for a couple of virtual folders,
// My Pictures and My Documents. 
//

static HRESULT WINAPI GetSpecialFolderPath( HWND hWndOwner, INT nFolder, LPTSTR lpFolder )
{
    LPMALLOC pMalloc  = NULL;
    HRESULT hResult   = NOERROR;
    BOOL bResult      = TRUE;
    LPITEMIDLIST pidl = NULL;

    SHGetMalloc (&pMalloc);
    hResult = SHGetSpecialFolderLocation ( hWndOwner, nFolder, &pidl );
    if ( SUCCEEDED( hResult ) )
        bResult = SHGetPathFromIDList ( pidl, lpFolder );
    if ( pMalloc != NULL )
    {
        if ( pidl != NULL )
        {
            pMalloc->Free( (LPVOID) pidl );
            pidl = NULL;
        }
        pMalloc->Release();
        pMalloc = NULL;
    }
    if ( bResult == FALSE )
        hResult = E_FAIL;
    return hResult;
}

//
// The normal way to use the config dialog is to first construct it, then
// set all the data members that correspond to INI entries. Once that is done,
// the DoModal() member is called, which results in this member being called
// right before the dialog is displayed. This is where all the sliders, 
// checkboxes, edit boxes, etc. are set to the appropriate values.
//
BOOL CCubeShowConfigDialog::OnInitDialog() 
{
    CDialog::OnInitDialog();
    
    //
    // There are seven sliders in the dialog, this is where they are all set to
    // have the correct ranges, positions, tick marks, etc.
    //
    SendDlgItemMessage( IDC_SLIDER_SPIN_X, TBM_SETRANGE, TRUE, MAKELONG( -99, 99 ) );
    SendDlgItemMessage( IDC_SLIDER_SPIN_X, TBM_SETTIC, 0, 0 );
    SendDlgItemMessage( IDC_SLIDER_SPIN_Y, TBM_SETRANGE, TRUE, MAKELONG( -99, 99 ) );
    SendDlgItemMessage( IDC_SLIDER_SPIN_Y, TBM_SETTIC, 0, 0 );
    SendDlgItemMessage( IDC_SLIDER_SPIN_Z, TBM_SETRANGE, TRUE, MAKELONG( -99, 99 ) );
    SendDlgItemMessage( IDC_SLIDER_SPIN_Z, TBM_SETTIC, 0, 0 );
    SendDlgItemMessage( IDC_SLIDER_SPEED_X, TBM_SETRANGE, TRUE, MAKELONG( 0, 99 ) );
    SendDlgItemMessage( IDC_SLIDER_SPEED_X, TBM_SETTIC, 0, 0 );
    SendDlgItemMessage( IDC_SLIDER_SPEED_Y, TBM_SETRANGE, TRUE, MAKELONG( 0, 99 ) );
    SendDlgItemMessage( IDC_SLIDER_SPEED_Y, TBM_SETTIC, 0, 0 );
    SendDlgItemMessage( IDC_SLIDER_SPEED_Z, TBM_SETRANGE, TRUE, MAKELONG( 0, 99 ) );
    SendDlgItemMessage( IDC_SLIDER_SPEED_Z, TBM_SETTIC, 0, 0 );
    SendDlgItemMessage( IDC_SLIDER_CUBE_SIZE, TBM_SETRANGE, TRUE, MAKELONG( 0, 99 ) );

    SendDlgItemMessage( IDC_SLIDER_SPIN_X, TBM_SETPOS,   TRUE, m_SliderSpinX );
    SendDlgItemMessage( IDC_SLIDER_SPIN_Y, TBM_SETPOS,   TRUE, m_SliderSpinY );
    SendDlgItemMessage( IDC_SLIDER_SPIN_Z, TBM_SETPOS,   TRUE, m_SliderSpinZ );
    SendDlgItemMessage( IDC_SLIDER_SPEED_X, TBM_SETPOS,   TRUE, m_SliderSpeedX );
    SendDlgItemMessage( IDC_SLIDER_SPEED_Y, TBM_SETPOS,   TRUE, m_SliderSpeedY );
    SendDlgItemMessage( IDC_SLIDER_SPEED_Z, TBM_SETPOS,   TRUE, m_SliderSpeedZ );
    SendDlgItemMessage( IDC_SLIDER_CUBE_SIZE, TBM_SETPOS,   TRUE, m_SliderCubeSize );
    SendDlgItemMessage( IDC_UPRIGHT_CHECK, BM_SETCHECK, m_KeepUpright ? BST_CHECKED : BST_UNCHECKED );
    //
    // Normally, the edit box with the image path shows the previously selected folder.
    // However, the very first time up, the INI file won't have a value for the image
    // folder. When that's the case, an attempt is made to initialize it with a pointer
    // to the "My Pictures" folder. If there isn't one, we then go to "My Documents."
    //
#ifndef CSIDL_MYPICTURES
#define CSIDL_MYPICTURES 0x27
#endif
    if ( m_PicturePath.GetLength() == 0 )
    {
        TCHAR szFolder[MAX_PATH];
        if ( GetSpecialFolderPath( NULL, CSIDL_MYPICTURES, szFolder ) == S_OK )
            SetDlgItemText( IDC_PICTURE_PATH, szFolder );
        else if ( GetSpecialFolderPath( NULL, CSIDL_PERSONAL, szFolder ) == S_OK )
            SetDlgItemText( IDC_PICTURE_PATH, szFolder );
    } else
        SetDlgItemText( IDC_PICTURE_PATH, m_PicturePath );

    //
    // Most of the controls in this dialog have tooltips. The tooltip control is created here,
    // and the messages for the various controls are initialized as well.
    // 
    m_tooltip.Create( this );
    m_tooltip.Activate( TRUE );
    char *msgx = "This slider controls the speed and direction of spin along the X axis";
    char *msgy = "This slider controls the speed and direction of spin along the Y axis";
    char *msgz = "This slider controls the speed and direction of spin along the Z axis";
    m_tooltip.AddTool( GetDlgItem( IDC_SLIDER_SPIN_X ), msgx );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_X1 ),     msgx );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_X2 ),     msgx );
    m_tooltip.AddTool( GetDlgItem( IDC_SLIDER_SPIN_Y ), msgy );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_Y1 ),     msgy );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_Y2 ),     msgy );
    m_tooltip.AddTool( GetDlgItem( IDC_SLIDER_SPIN_Z ), msgz );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_Z1 ),     msgz );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_Z2 ),     msgz );
    msgx = "This slider controls the speed of movement along the X axis";
    msgy = "This slider controls the speed of movement along the Y axis";
    msgz = "This slider controls the speed of movement along the Z axis";
    m_tooltip.AddTool( GetDlgItem( IDC_SLIDER_SPEED_X ), msgx );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_X ),       msgx );
    m_tooltip.AddTool( GetDlgItem( IDC_SLIDER_SPEED_Y ), msgy );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_Y ),       msgy );
    m_tooltip.AddTool( GetDlgItem( IDC_SLIDER_SPEED_Z ), msgz );
    m_tooltip.AddTool( GetDlgItem( IDC_STATIC_Z ),       msgz );
    char *msgsize = "This slider controls the size of the cube";
    m_tooltip.AddTool( GetDlgItem( IDC_SLIDER_CUBE_SIZE ), msgsize );
    char *msgupright = "If this checkbox is set, CubeShow will attempt to keep images upright as the cube rotates";
    m_tooltip.AddTool( GetDlgItem( IDC_UPRIGHT_CHECK ), msgupright );
    char *msgpath = "This edit control contains the folder that will be scanned for image files";
    m_tooltip.AddTool( GetDlgItem( IDC_PICTURE_PATH ), msgpath );
    char *msgbrowse = "Selecting this button will allow you to browse for a folder that contains image files";
    m_tooltip.AddTool( GetDlgItem( IDC_SELECT_DIRECTORY_BUTTON ), msgbrowse );

    return TRUE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
}

//
// As the dialog is being destroyed, I update all the member values.
// This could probably be done just as well or perhaps better using
// MFC's DDX functions, but this way is not bad if you are a control freak.
//
void CCubeShowConfigDialog::OnDestroy() 
{
    CDialog::OnDestroy();
    m_SliderSpinX = SendDlgItemMessage( IDC_SLIDER_SPIN_X, TBM_GETPOS );
    m_SliderSpinY = SendDlgItemMessage( IDC_SLIDER_SPIN_Y, TBM_GETPOS );
    m_SliderSpinZ = SendDlgItemMessage( IDC_SLIDER_SPIN_Z, TBM_GETPOS );
    m_SliderSpeedX = SendDlgItemMessage( IDC_SLIDER_SPEED_X, TBM_GETPOS );
    m_SliderSpeedY = SendDlgItemMessage( IDC_SLIDER_SPEED_Y, TBM_GETPOS );
    m_SliderSpeedZ = SendDlgItemMessage( IDC_SLIDER_SPEED_Z, TBM_GETPOS );
    m_SliderCubeSize = SendDlgItemMessage( IDC_SLIDER_CUBE_SIZE, TBM_GETPOS );
    m_KeepUpright = SendDlgItemMessage( IDC_UPRIGHT_CHECK, BM_GETCHECK ) == BST_CHECKED;
    GetDlgItemText( IDC_PICTURE_PATH, m_PicturePath );
    if ( m_SliderSpinX >= -6 && m_SliderSpinX <= 6 ) m_SliderSpinX = 0;
    if ( m_SliderSpinY >= -6 && m_SliderSpinY <= 6 ) m_SliderSpinY = 0;
    if ( m_SliderSpinZ >= -6 && m_SliderSpinZ <= 6 ) m_SliderSpinZ = 0;
    if ( m_SliderSpeedX >= -6 && m_SliderSpeedX <= 6 ) m_SliderSpeedX = 0;
    if ( m_SliderSpeedY >= -6 && m_SliderSpeedY <= 6 ) m_SliderSpeedY = 0;
    if ( m_SliderSpeedZ >= -6 && m_SliderSpeedZ <= 6 ) m_SliderSpeedZ = 0;
}

//
// When browsing for a new image folder, this dialog uses the SHBrowseForFolder()
// function. By specifying a callback function I can customize the way the 
// browse dialog works. This callback routine has two particular things that it
// does. First, when the browse dialog is fully up, the callback routine forces
// it to jump to the current picture directory. Second, if the current folder
// looks like the Control Panel, I disable the OK button.
//

int CALLBACK CCubeShowConfigDialog::BrowseCallbackProc( HWND hwnd, 
                                                        UINT uMsg, 
                                                        LPARAM lParam, 
                                                        LPARAM lpData )
{
    if ( uMsg == BFFM_INITIALIZED )
    {
        CString folder;
        ((CCubeShowConfigDialog *)lpData)->GetDlgItemText( IDC_PICTURE_PATH, folder );
        ::SendMessage( hwnd, BFFM_SETSELECTION, TRUE, (LPARAM) (const char *) folder );
    } else if ( uMsg == BFFM_SELCHANGED )
    {
        ITEMIDLIST *itemList = (ITEMIDLIST *) lParam;
        if ( itemList )
        {
            char path[ _MAX_PATH ];
            if ( SHGetPathFromIDList( itemList, path ) )
            {
                CString cpath = path;
                CString match = "{21EC2020-3AEA-1069-A2DD-08002B30309D}";
                cpath = cpath.Right( match.GetLength() );
                if ( cpath.Compare( match ) == 0 )
                {
                    ::SendMessage( hwnd, BFFM_ENABLEOK, 0, 0 );
                }
            }
        }
    }
    return 0;
}

//
// If the user wants to select a new folder, they press the button that
// invokes this routine. The user exits this routine by either pressing
// OK or Cancel. If they press OK, we take note of that by setting the
// flag that will force a conversion.
//
void CCubeShowConfigDialog::OnSelectDirectoryButton() 
{
    char displayName[ _MAX_PATH ];
    BROWSEINFO browseInfo =
    {
        this->m_hWnd,
        NULL,
        displayName,
        "Select a folder containing image files of type JPG, DIB, WMF, or BMP :",
        BIF_RETURNONLYFSDIRS, // + 0x40 for new style
        BrowseCallbackProc,
        (long) this,
        0 
    };
    ITEMIDLIST *itemList = SHBrowseForFolder( &browseInfo );
    if ( itemList )
    {
        char path[ _MAX_PATH ];
        if ( SHGetPathFromIDList( itemList, path ) )
        {
          SetDlgItemText( IDC_PICTURE_PATH, path );
          m_ForceConvert = true;
        }
    }
}

//
// Pressing the About button brings up the About box. 
//
void CCubeShowConfigDialog::OnAbout() 
{
  CAboutBox aboutBox;
  aboutBox.DoModal();
}

//
// A member function that was added when using Visual Studio's tooltip component.
// The "CG" initials signify mods made by the Component Gallery.
//
BOOL CCubeShowConfigDialog::PreTranslateMessage(MSG* pMsg)
{
    // CG: The following block was added by the ToolTips component.
    {
        // Let the ToolTip process this message.
        m_tooltip.RelayEvent(pMsg);
    }
    return CDialog::PreTranslateMessage(pMsg);    // CG: This was added by the ToolTips component.
}

//
// When the user is typing in a new path in the image area, I keep a close eye on things.
// I check to see if the path is valid, and if it isn't, I disable the OK button.
//
void CCubeShowConfigDialog::OnChangePicturePath() 
{
  CString path;
  GetDlgItemText( IDC_PICTURE_PATH, path );
  if ( path.Right( 0 ).Compare( "/" ) != 0 || path.Right( 0 ).Compare( "\\" ) != 0 )
    path += "/";
  if ( _access( path + ".", 0 ) == -1 )
    m_OkButton.EnableWindow( FALSE );
  else
    m_OkButton.EnableWindow( TRUE );
}

//
// When the user presses OK, the settings dialog is basically done. All of the INI file
// properties are going to be updated by the caller, so we don't worry about that here.
// The only important piece of work left to do at this point is to manage the image list.
// If it appears that the user has selected a new image path, it means that we have to
// go through the conversion process, which means copying all the images in the image path
// to reduced sized BMP files in the image cache.
//
void CCubeShowConfigDialog::OnOK() 
{
    CString path;
    GetDlgItemText( IDC_PICTURE_PATH, path );
    int oldCount = AfxGetApp()->GetProfileInt( "Photos", "Count", 0 );
    CConvert dlg;
    dlg.SetSourcePath( path );
    bool convert = AfxGetApp()->GetProfileInt( "Settings", "Convert", 1 ) ? true : false ;
    dlg.SetConvert( convert );
    int targetBmpSize = AfxGetApp()->GetProfileInt( "ConversionParms", "TargetBmpSize", 256 );
    dlg.SetTargetBmpSize( targetBmpSize );
    //
    // Everything you've seen above here is all setup in preparation for launching the convert
    // dialog. At this point, we have to decide whether to actually launch it. We will launch
    // it if any of three conditions are true: 1) m_ForceConvert is set if the user selects a folder
    // from the Browse button (even if it's the same folder.) 2) The image path has changed. 
    // 3) The number of images in the image path doesn't agree with the count in the INI file.
    //
    if ( m_ForceConvert || path.Compare( m_PicturePath ) != 0 || oldCount != dlg.GetInputCount() )
    {
        CString cachePath = ((CCubeShowApp *) AfxGetApp())->m_AppPath + "ImageCache\\";
        char cachePathBuff[ _MAX_PATH ];
        char newPathBuff[ _MAX_PATH ];
        char *p;
        DWORD result = GetFullPathName( path, _MAX_PATH, newPathBuff, &p );
        result = GetFullPathName( cachePath, _MAX_PATH, cachePathBuff, &p );
        if (strcmpi( cachePathBuff, newPathBuff ) == 0 )
        {
            MessageBox( "It appears that you have selected your CubeShow's Image Cache folder as the source of your image files."
                        " CubeShow is unable to operate in this configuration. Please select a different folder." );
            return;
        }     
        //
        // At this point I am obligated to delete all the bmp files in the preview directory
        //
        DeleteImageCacheFiles();
        //
        // I am also obligated to delete all the entries in the INI file
        //
        int count = AfxGetApp()->GetProfileInt( "Photos", "Count", 0 );
        for ( int i = 0; i < count ; i++ )
        {
          CString entry;
          entry.Format( "Entry%d", i );
          AfxGetApp()->WriteProfileString( "Photos", (const char *) entry, NULL );
        }
        //
        // Now I write a temporary count of 0 just in case things fall apart during conversion.
        //
        AfxGetApp()->WriteProfileInt( "Photos", "Count", 0 );
        //
        // Finally! I call the conversion dialog to do its work. Once it completes, I have
        // a list of new files. I update the INI file with the new count, as well as with 
        // each of the new files.
        //
        if ( dlg.DoModal() == IDOK )
        {
          count = dlg.GetProcessedCount();
          Trace( "Old count = %d, new count = %d\n", oldCount, count );
          CString msg;
          msg.Format( "Processed %d image files", count );
          Trace( "%s\n", msg );
          MessageBox( msg, "CubeShow", MB_OK );
          AfxGetApp()->WriteProfileInt( "Photos", "Count", count );
          for ( int i = 0 ; i < count ; i++ )
          {
              CString entry;
              entry.Format( "Entry%d", i );
              AfxGetApp()->WriteProfileString( "Photos", entry, dlg.GetOutputFileName( i ) );
          }
        } else
            return;
    }
    CDialog::OnOK();
}

//
// This utility function deletes all the BMP files from the image cache.
//
void CCubeShowConfigDialog::DeleteImageCacheFiles()
{
    CString outputFile = ((CCubeShowApp *) AfxGetApp())->m_AppPath + "ImageCache\\*.bmp";
    CFileFind finder;
    BOOL bWorking = finder.FindFile( outputFile );
    while ( bWorking )
    {
        bWorking = finder.FindNextFile();
        CString path = finder.GetFilePath();
        //
        // Deleting files is scary - so I'm going to take a few precautions
        //
        if ( path.Find( "\\ImageCache\\" ) != -1 && path.Right( 4 ).CompareNoCase( ".bmp" ) == 0 )
        {
            Trace( "Deleting file %s\n", path );
            ::DeleteFile( path );
        } else {
            Trace( "INCORRECT ATTEMPT TO DELETE %s\n", path );
        }
    }
}

void CCubeShowConfigDialog::OnHelp() 
{
    CString filename = ((CCubeShowApp *) AfxGetApp())->m_AppPath + "CubeShow.hlp";
//    ::WinHelp( m_hWnd, filename, HELP_FINDER, 0 );
    ::WinHelp( m_hWnd, filename, HELP_CONTEXT, 4 );	
}

void CCubeShowConfigDialog::OnAdvanced() 
{
    //
    // The Advanced dialog returns TRUE or FALSE. If it returns TRUE, it means we need to resample all
    // the image files, because the Target BMP Size has changed.
    //
    CAdvancedDialog dlg;
    if ( dlg.DoModal() == TRUE )
        m_ForceConvert = TRUE;
}
