//
//   DXGraphics.h : Declaration file for class DXGraphics
//
//  CubeShow Version 1.0 Copyright (c) 2002
//  Ziff Davis Media, Inc.
//  All rights reserved.
//  First Published in PC Magazine, US Edition, July 2000
//
//  Author : Mark Nelson
//  Date   : May 23, 2002
//
//  Description
//
//  DXGraphics is the class that sets up all of the necessary parts of DirectX,
//  then draws the cube to the screen upon request. I learned most of what I
//  needed to write this code from the tutorials on Ken Paulson's web site at
//  http://www.drunkenhyena.com.
//

//

#ifndef _dxgraphics_h_
#define _dxgraphics_h_


#include <d3d8.h>
#include <d3dx8math.h>
#include <dxerr8.h>

class DXGraphics 
{
    // vertex_t includes texture coords
    struct vertex_t {
        float x,y,z;
        float tu,tv;
    };

    // the FVF code that matches our vertex_t struct
    enum { vertexFormat = D3DFVF_XYZ | D3DFVF_TEX1 };

    struct tvertex_t {
        float x,y,z;
        float rhw;
        float tu,tv;
    };
    enum { tvertexFormat = D3DFVF_XYZRHW | D3DFVF_TEX1 };

public:
    DXGraphics( HINSTANCE );
    ~DXGraphics();

    void create();
    void iterate();
    void setScreenMode( int width, int height, bool windowed )
    {
        scrWidth = width;
        scrHeight = height;
        scrWindowed = windowed;
    }
    void setRotations( int x, int y, int z )
    {
        xRotStep = x * 3.14159f / 180.0f / 2;
        yRotStep = y * 3.14159f / 180.0f / 2;
        zRotStep = z * 3.14159f / 180.0f / 2;
    }
    void setTranslations( int x, int y, int z )
    {
        xTransStep = (float) x / 2500.0f / 2;
        yTransStep = (float) y / 10000.0f / 2;
        zTransStep = (float) z / 6000.0f / 2;
    }
    void setSize( int val )
    {
        zMin = 5.1f - val * ( 3.0f / 99.0f );
        zMax = 10.1f - val * ( 7.0f / 99.0f );
    }
    void setForceUpright( bool val )
    {
        forceUpright = val;
    }
    void setLowScreenResolution( bool val )
    {
        lowScreenResolution = val;
    }

private:
    void createWindow();
    void cleanup();
    bool createDX();
    void createCube();
    void createCube_original();
    void loadTexture();
    void makeRightSideUp( D3DXMATRIX *matrix, int face );

    IDirect3D8* graphics;
    IDirect3DDevice8* device;
    IDirect3DVertexBuffer8* cube;

public :
    bool good();
    const char *error(){ return ErrorString; }
    CWinThread * textureLoaderThread;
    deque<IDirect3DTexture8*> queue;
    CCriticalSection queueLock;
    HANDLE killEvent;
    HWND graphWindow;
    bool scrWindowed;
    int scrWidth;
    int scrHeight;
    bool lowScreenResolution;
private :
    char *ErrorString;
    HINSTANCE appInstance;

    IDirect3DTexture8* texture[ 6 ];
    bool picturePresent[ 6 ];
    float xRotStep;
    float yRotStep;
    float zRotStep;
    float xTransStep;
    float yTransStep;
    float zTransStep;
    float zMax;
    float zMin;
    bool forceUpright;
private:
    DWORD startTime;
    int makeRight;
    float x;
    float y;
    float z;

protected:
    D3DFORMAT getGoodFormat( D3DFORMAT currentFormat, D3DDEVTYPE device, const CString& name  );
    int faceToRotate;
    static UINT TextureLoader( LPVOID lParam );
};
#endif
