//
//  Trace.cpp : Implementation file for class Trace
//
//  CubeShow Version 1.1 Copyright (c) 2002
//  Ziff Davis Media, Inc.
//  All rights reserved.
//  First Published in PC Magazine, US Edition, July 2000
//
//  Author : Mark Nelson
//  Date   : September 29, 2002
//
//  Description
//
//  The Trace class is used to capture debug output from CubeShow.
//  Information to be traced is scattered throughout the program via
//  calls to Trace(), which appear to be normal function calls, but
//  are actually constructors to the Trace object. The arguments to
//  the Trace constructor have the same syntax as calls to printf(),
//  and have the same semantics. 
//
//  The first time the Trace() constructor is invoked, it checks to 
//  see if the Debug Logger is up and running. If it is, the Window
//  handle is capture, and all Trace output will be sent to that window
//  from then on. If the Window doesn't exist at that point, all Trace
//  output will be disgarded for the remainder of the program.
//


#include "stdafx.h"
#include "Trace.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

bool Trace::FirstTime   = true;
HWND Trace::traceTarget = NULL;

Trace::Trace(const char *fmt, ... )
{
    if ( FirstTime )
        FindWindow();
#if !defined( _DEBUG )
    if ( traceTarget == NULL )
        return;
#endif

    va_list args;
	va_start( args,  fmt);

	int len;
	char buf[512];

	len = _vsnprintf( buf, sizeof(buf) -1, fmt, args);
    buf[ 511 ] = 0;
	va_end(args);
    if ( traceTarget != NULL ) 
    {
        CString temp( buf );
        temp.Replace( "\n", "\r\n" );

        COPYDATASTRUCT data;
        data.cbData = temp.GetLength() + 1;
        data.lpData = (void *)(const char *) temp;
        data.dwData = 0;
        ::SendMessage( traceTarget, WM_COPYDATA, 0, (LPARAM) &data );
    } 
    else 
      OutputDebugString( buf );
}

void Trace::FindWindow()
{
    FirstTime = false;
    traceTarget = ::FindWindow( NULL, "CubeShow Debug Logger" );
}
